/* find-suffix.c: return the stuff after a dot.

   Copyright 1992, 1993, 1995, 2008 Karl Berry.

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or (at your option) any later version.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.

   You should have received a copy of the GNU Lesser General Public License
   along with this library; if not, see <http://www.gnu.org/licenses/>.  */

#include <kpathsea/config.h>

#include <kpathsea/c-pathch.h>

#if defined(WIN32) && !defined(__MINGW32__)
#include <kpathsea/knj.h>
#endif

/* Return pointer to first character after `.' in last directory element
   of NAME.  If the name is `foo' or `/foo.bar/baz', we have no extension.  */

string
find_suffix (const_string name)
{
#if defined(WIN32) && !defined(__MINGW32__)
  string p;
  string dot_pos;
  const_string slash_pos;
  const_string orig_name;
  string strname;
  orig_name = name;
  strname = (string) xstrdup (name);
  for(p=strname; *p; p++) {
    if(isknj(*p) && isknj2(*(p+1))) {
      p++;
      continue;
    }
    if(*p == '\\') *p = '/';
  }
  name = (const_string) strname;
  dot_pos = strrchr (name, '.');
#else
  const_string slash_pos;
  string dot_pos = strrchr (name, '.');
#endif

#if defined(WIN32) && !defined(__MINGW32__)
  if (dot_pos == NULL) {
    name = orig_name;
    free (strname);
    return NULL;
  }
#else
  if (dot_pos == NULL)
    return NULL;
#endif

  for (slash_pos = name + strlen (name);
#if defined(WIN32) && !defined(__MINGW32__)
       slash_pos > dot_pos && (*slash_pos != '/');
#else
       slash_pos > dot_pos && !IS_DIR_SEP (*slash_pos);
#endif
       slash_pos--)
    ;
#if defined(WIN32) && !defined(__MINGW32__)
  name = orig_name;
  if (slash_pos > dot_pos) p = NULL;
  else p = (string) (name + (dot_pos - strname + 1));
  free (strname);
  return p;
#else
  return slash_pos > dot_pos ? NULL : dot_pos + 1;
#endif
}
