/* 
   special version for ConTeXt
   (2007/08/11 --ak)
   (2009/04/29 --ak)
*/

#include <windows.h>
#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <malloc.h>
#include <process.h>
#include <kpathsea/kpathsea.h>

#define SBUF    256
#define MAXARG  512

char *getlname(char *name)
{
  HANDLE hnd;
  WIN32_FIND_DATA ffd;

  char *path = NULL;
  char buff[SBUF];
  char buff2[SBUF];
  char *p, *fp;

  strcpy(buff, name);
  for(p = buff; *p; p++) {
    if(isknj(*p)) {
      if(*(p+1)) p++;
      continue;
    }
    if(*p == '\\') *p = '/';
  }
  fp = buff;
  if(p = kstrrchr(fp, '/')) {
    p++;
    fp = p;
  }
  if(p = kstrrchr(fp, ':')) {
    p++;
    fp = p;
  }
  if(p = kstrrchr(fp, '.')) *p = '\0';
  strcpy(buff2, fp);

  if(SearchPath((char *)getenv("PATH"), buff2, ".exe", SBUF, buff, &fp)) {
    if ((hnd = FindFirstFile(buff, &ffd)) == NULL) {
      return NULL;
    }
    FindClose(hnd);

    path = (char *)xmalloc(strlen(buff) + 1);
    strcpy(path, buff);
    for(p=path; *p; p++) {
      if(isknj(*p)) {
	if(*(p+1)) p++;
        continue;
      }
      if(*p == '\\') *p = '/';
    }
    if(p = kstrrchr(path, '/')) *p = '\0';

  }
  else {
    return NULL;
  }
  return path;
}

/* search for the script in the TeX bin directory */
int searchscr1(char *path, char *name, char *ext, char *dstname)
{
  char *fname;
  int  ret = 0;
  fname = concatn(path, "/", name, ext, NULL);
  if(_access(fname, 0) == 0) {
    strcpy(dstname, fname);
    ret = 1;
  }
  free(fname);
  return ret;
}

/* search for the script under $TEXMF/scripts// */
int searchscr2(char *name, char *ext, char *dstname, int isluap)
{
  char *fname, *fullname;
  int  ret = 0;
  fname = concat(name, ext);
  fullname = kpse_find_file(fname, kpse_texmfscripts_format, 0);
  if(!fullname && isluap) {
    fullname = kpse_find_file(fname, kpse_lua_format, 0);;
  }
  if(fullname) {
    strcpy(dstname, fullname);
    free(fullname);
    ret = 1;
  }
  free(fname);
  return ret;
}

int main(int ac, char **av)
{
  int i, ret, trees;
  char *argv[MAXARG];
  char *p;
  char scrname[SBUF];
  char fname[SBUF];
  FILE *f;
  char *fullname;
  char cmdname[SBUF];
  char fullbin[SBUF];

  char *t1, *t2, *t3, *t4;
  int  kind;

  av[0] = (char *)getlongname(av[0]);
  kpse_set_program_name(av[0], NULL);
  cmdname[0] = '\0';
  trees = 0;

  if(ac>MAXARG - 5) {
    fprintf(stderr, "Too many arguments.\n");
    exit (100);
  }

  for(i=0; i < ac + 4; i++)
    argv[i] = (char *)malloc(SBUF);

  p = getlname(av[0]);
  if(!p) {
    fprintf(stderr, "Failed to find script loader.\n");
    for(i=0; i < ac + 2; i++)
      free(argv[i]);
    exit (100);
  }

/* env variables for ConTeXt MKIV */

/* OSFONTDIR */
  t1 = getenv("SYSTEMROOT");
  if(t1) {
     t1 = xstrdup(t1);
     for(t2=t1; *t2; t2++) {
	if(*t2 == '\\') *t2 = '/';
     }
     t2 = (char *)xmalloc(strlen(t1)+strlen("/fonts//")+2);
     strcpy(t2, t1);
     strcat(t2, "/fonts//");
     xputenv("OSFONTDIR", t2);
     free(t1);
     free(t2);
  } else {
     xputenv("OSFONTDIR", "c:/windows/fonts//");
  }

/* TEXMFCACHE */
  t1 = NULL;
  t2 = kpse_var_value("TEXMFCACHE");
  if(t2) {
    t1 = kpse_brace_expand(t2);
    free(t2);
    t2 = strchr(t1, ';');
    if(t2) *t2 = '\0';
  }
  if(!t1) {
    t1 = getenv("TEMP");
    if(!t1) t1 = getenv("TMP");
    if(!t1) t1 = getenv("TMPDIR");
    if(!t1) {
      fprintf(stderr, "Define TEMP or TMP or TMPDIR\n");
      exit(100);
    }
    t1 = xstrdup(t1);
  }
  for(t2=t1; *t2; t2++) {
     if(isknj(*t2)) {
	if(*(t2+1)) t2++;
	continue;
     }
     if(*t2 == '\\') *t2='/';
  }
  xputenv("TEXMFCACHE", t1);
  free(t1);

/* TEXMFCNF */
  t1 = kpse_var_value("TEXMFCNF");
  if(!t1) {
     fprintf(stderr, "I cannot determine TEXMFCNF\n");
     exit(100);
  }
  t2 = kpse_brace_expand(t1);
  free(t1);
  t1 = (char *)xmalloc(strlen(t2) + 3);
  t3 = t1;
  *t3++ = '{';
  for(t4=t2; *t4; t4++) {
     if(isknj(*t4)) {
	if(*(t4+1)) t4++;
	continue;
     }
     if(*t4 == '!')
	continue;
     if(*t4 == ';') {
	*t3++ = ',';
	continue;
     }
     *t3++ = *t4;
  }
  *t3++ = '}';
  *t3 = '\0';
  xputenv("TEXMFCNF", t1);
  free(t1);
  free(t2);

/* TEXMF */
  t1 = kpse_var_value("TEXMF");
  if(!t1) {
     fprintf(stderr, "I cannot determine TEXMF\n");
     exit(100);
  }
  t2 = kpse_brace_expand(t1);
  free(t1);
  t1 = (char *)xmalloc(strlen(t2) + 3 + 32 * 2); /* at most 32 trees */
  t3 = t1;
  *t3++ = '{';
  *t3++ = '!'; /* force cached or !! in context (2010 --ak) */
  *t3++ = '!';
  trees++;
  for(t4=t2; *t4; t4++) {
     if(isknj(*t4)) {
	if(*(t4+1)) t4++;
	continue;
     }
     if(*t4 == '!')
	continue;
     if(*t4 == ';' || *t4 == ',') {
        trees++;
        if(trees > 32) {
           fprintf(stderr, "Too many texmf trees.\n");
           exit(3);
	}
	*t3++ = ',';
	*t3++ = '!'; /* force cached or !! in context (2010 --ak) */
	*t3++ = '!';
	continue;
     }
     *t3++ = *t4;
  }
  *t3++ = '}';
  *t3 = '\0';
  xputenv("TEXMF", t1);
  free(t1);
  free(t2);

  scrname[0] = '\0';

  if((strnicmp(av[0], "context", 7) == 0) ||

     (strnicmp(av[0], "metatex", 7) == 0) ||
     (strnicmp(av[0], "mptopdf", 7) == 0))
    kind = 1;
  else if(strnicmp(av[0], "luatools", 8) == 0) {
    kind = 1;
    strcpy(av[0], "base");
  }
  else if(strnicmp(av[0], "mtxrun", 6) == 0)
    kind = 2;
  else
    kind = 0;

  if(kind == 0) {
/* ruby (suffix .rb) */
    if(searchscr1(p, av[0], ".rb", scrname))
      goto Found;

/* lua (suffix .lua) */
    if(searchscr1(p, av[0], ".lua", scrname))
      goto Found;

    if(searchscr1(p, av[0], ".tlu", scrname))
      goto Found;

    if(searchscr1(p, av[0], ".texlua", scrname))
      goto Found;

/* python (suffix .py) */
    if(searchscr1(p, av[0], ".py", scrname))
      goto Found;

/* perl (suffix .pl) */
    if(searchscr1(p, av[0], ".pl", scrname))
      goto Found;

/* search script under $TEXMF/scripts */

/* ruby */
    if(searchscr2(av[0], ".rb", scrname, 0))
      goto Found;

/* lua */
    if(searchscr2(av[0], ".lua", scrname, 1))
      goto Found;

    if(searchscr2(av[0], ".tlu", scrname, 1))
      goto Found;

    if(searchscr2(av[0], ".texlua", scrname, 1))
      goto Found;

/* python */
    if(searchscr2(av[0], ".py", scrname, 0))
      goto Found;

/* perl */
    if(searchscr2(av[0], ".pl", scrname, 0))
      goto Found;

    if(!scrname[0]) {
      fprintf(stderr, "Failed to find script.\n");
      for(i=0; i < ac + 2; i++)
        free(argv[i]);
      exit (100);
    }
  }

 Found:

/* mtxrun.lua */
  fullname = kpse_find_file("mtxrun", kpse_lua_format, 0);
  if(fullname == NULL) {
    fullname = kpse_find_file("mtxrun.lua", kpse_texmfscripts_format, 0);
  }

  if(fullname == NULL) {
    fprintf(stderr, "I cannot find mtxrun.lua.\n");
    exit (2);
  }

  strcpy(cmdname, fullname);
  free(fullname);

  if(kind == 1) {
    strcpy(argv[0], "texlua.exe");
    strcpy(argv[1], cmdname);
    strcpy(argv[2], "--script");
    strcpy(argv[3], av[0]);
    for(i=1; i < ac; i++)
      strcpy(argv[i+3], av[i]);
    argv[ac+3] = NULL;
  } else if(kind == 2) {
    strcpy(argv[0], "texlua.exe");
    strcpy(argv[1], cmdname);
    for(i=1; i < ac; i++)
      strcpy(argv[i+1], av[i]);
    argv[ac+1] = NULL;
  } else {
    strcpy(argv[0], "texlua.exe");
    strcpy(argv[1], cmdname);
    strcpy(argv[2], "--usekpse");
    strcpy(argv[3], "--execute");
    strcpy(argv[4], scrname);
    for(i=1; i < ac; i++)
      strcpy(argv[i+4], av[i]);
    argv[ac+4] = NULL;
  }

  p = kpse_var_value("SELFAUTOLOC");
  strcpy(fullbin, p);
  strcat(fullbin, "/");
  strcat(fullbin, "texlua.exe");
  free(p);

/*
  for(i=0; i < ac + 4; i++)
    fprintf(stderr, "%s ", argv[i]);
  fprintf(stderr, "\n");
  return (0);
*/

  ret = spawnv(_P_WAIT, fullbin, argv);

  for(i=0; i < ac + 4; i++)
    free(argv[i]);
  return ret;
}
